---------------------------------------------------------------
-- Copyright Esperan Multimedia Products Limited 1995
--
-- Code for the array based display driver for the Alarm Clock 
-- project, forming part of the MasterClass VHDL Tutorial. 
--------------------------------------------------------------- 

Library IEEE;
use IEEE.Std_Logic_1164.all; 

entity DDRV4 is
port ( ALARM_TIME_LS_MIN, CURRENT_TIME_LS_MIN  : in std_logic_vector(3 downto 0);
       ALARM_TIME_MS_MIN, CURRENT_TIME_MS_MIN  : in std_logic_vector(3 downto 0);
       ALARM_TIME_LS_HR, CURRENT_TIME_LS_HR    : in std_logic_vector(3 downto 0);
       ALARM_TIME_MS_HR, CURRENT_TIME_MS_HR    : in std_logic_vector(3 downto 0);
       SHOW_A                    : in std_logic;
       SOUND_ALARM               : out std_logic;
       DISPLAY_LS_MIN            : out std_logic_vector(6 downto 0);
       DISPLAY_MS_MIN            : out std_logic_vector(6 downto 0);
       DISPLAY_LS_HR             : out std_logic_vector(6 downto 0);
       DISPLAY_MS_HR             : out std_logic_vector(6 downto 0)
     );
end DDRV4;  

architecture RTL of DDRV4 is

  component DDRV
    port ( ALARM_TIME, CURRENT_TIME  : in std_logic_vector (3 downto 0);
               SHOW_A                    : in std_logic;
               SOUND_ALARM               : out std_logic;
               DISPLAY                   : out std_logic_vector (6 downto 0)
     );
  end component;

  signal LS_MIN_SOUND_ALARM : std_logic;
  signal MS_MIN_SOUND_ALARM : std_logic;
  signal LS_HR_SOUND_ALARM : std_logic;
  signal MS_HR_SOUND_ALARM : std_logic;
  
begin

  LS_MIN : DDRV port map ( ALARM_TIME_LS_MIN,
                           CURRENT_TIME_LS_MIN,
                           SHOW_A,
                           LS_MIN_SOUND_ALARM,
                           DISPLAY_LS_MIN );
  
  MS_MIN : DDRV port map ( ALARM_TIME_MS_MIN,
                           CURRENT_TIME_MS_MIN,
                           SHOW_A,
                           MS_MIN_SOUND_ALARM,
                           DISPLAY_MS_MIN );

  LS_HR : DDRV port map ( ALARM_TIME_LS_HR,
                  CURRENT_TIME_LS_HR,
                  SHOW_A,
                  LS_HR_SOUND_ALARM,
                  DISPLAY_LS_HR );

  MS_HR : DDRV port map ( ALARM_TIME_MS_HR,
                  CURRENT_TIME_MS_HR,
                  SHOW_A,
                  MS_HR_SOUND_ALARM,
                  DISPLAY_MS_HR );

  SOUND_ALARM <= (LS_MIN_SOUND_ALARM and MS_MIN_SOUND_ALARM) and (LS_HR_SOUND_ALARM and MS_HR_SOUND_ALARM);

end RTL ;


